# Correctness proof for the Hashgraph Consensus algorithm, formalized in Coq.

A formalization of *The Swirlds Hashgraph Consensus Algorithm: Fair,
Fast, Byzantine Fault Tolerance*, by Leemon Baird.  Formalized in Coq
by Karl Crary, using some insights arising from an earlier,
partial formalization by Gregory Malecha and Ryan Wisnesky.

## Build Instructions:

The development is self-contained and should build without
modification using Coq version 8.8.0.  Just run make.

## Contents (in dependency order):

* Tact.v: Tactics used by the development.

* Relation.v : Definitions and lemmas regarding relations.

* Decide.v: Definitions and lemmas regarding decidability.

* Cardinality.v: Cardinality of finite sets.

* Calculate.v: Useful arithmetic facts.

* Majority.v: Majorities and supermajorities of finite sets.

* Median.v: The definition of median, and lemmas regarding it.

* Hashgraph.v: The types and axioms that underlie the Hashgraph algorithm.

* HashgraphFacts.v: Basic facts regarding hashgraphs.

* Sees.v: Sees and strongly-sees. The strongly seeing lemma.

* Round.v: Rounds created and witnesses.

* Progress.v: The algorithm continues to advance in round.

* Vote.v: Voting. Votes are consistent.

* Decision.v: Decisions. Decisions are consistent and propagate everywhere.

* Consensus.v: Consensus is eventually achieved on the fame of every witness.

* Famous.v: Famous witnesses exist, and no late-arriving event can be famous.

* Received.v: Unique famous witnesses and round received. The round received is defined, immutable, and eventually computable by every peer.

* Order.v: The consensus order. Events from honest peers eventually appear in the order. Once an event is placed in the order, its predecessors are determined permanently.

* Fairness.v: A supermajority of peers contribute a famous witness every round. Consequently, timestamps are determined by honest peers.
